package com.jdblabs.timestamper.cli

import com.jdbernard.io.NonBlockingInputStreamReader
import com.jdbernard.util.SmartConfig
import com.jdbernard.util.LightOptionParser
import com.jdblabs.timestamper.core.Timeline
import com.jdblabs.timestamper.core.TimelineMarker
import com.jdblabs.timestamper.core.TimelineProperties
import com.martiansoftware.nailgun.NGContext

import org.fusesource.jansi.AnsiConsole
import static org.fusesource.jansi.Ansi.*

import static org.fusesource.jansi.Ansi.*
import static org.fusesource.jansi.Ansi.Color.*

public class TimeStamperCLI {

    private static String EOL = System.getProperty("line.separator")

    private static TimeStamperCLI nailgunInst

    protected TimelineProperties timelineProperties
    protected Timeline timeline

    public static final String VERSION = "0.1"

    protected static def cli = [
        'v': [longOpt: 'version'],
        'd': [longOpt: 'working-directory', arguments: 1],
        't': [longOpt: 'timeline-config', arguments: 1] ]

    public static void main(String[] args) {

        TimeStamperCLI inst = new TimeStamperCLI()
        doMain(inst, args, System.in, System.out, System.err); }

    public static void nailMain(NGContext context) {
        if (nailgunInst == null)
            nailgunInst = new TimeStamperCLI()

        doMain(nailgunInst, context.args, context.in, context.out, context.err) }

    protected static doMain(TimeStamperCLI inst, String[] args,
    def sin, def out, def err) {

        //out = new PrintStream(AnsiConsole.wrapOutputStream(out))
        def opts = LightOptionParser.parseOptions(cli, args as List)

        File workingDir = new File(opts.d ?: '.')

        if (opts.t) {
            File propFile = new File(workingDir, opts.t)
            inst.showTimeline(propFile, sin, out, err) }
        else if (inst.timeline == null) {
            // Look for .timestamperrc user config file
            File cfgFile = new File(
                System.getProperty('user.home'), ".timestamperrc")
            if (!cfgFile.exists() || !cfgFile.isFile())
                err.println "Could not find the user configuration file: " +
                    cfgFile.canonicalPath
            else {
                def cfg = new SmartConfig(cfgFile)
                inst.showTimeline(new File(cfg.lastUsed), sin, out, err) } }
        else { inst.showTimeline(sin, out, err) } }

    public TimeStamperCLI() { }

    public void showTimeline(File timelinePropertiesFile,
    def sin, def out, def err) {
        if (!timelinePropertiesFile.exists() ||
            !timelinePropertiesFile.isFile()) {

            err.println "No such timeline property file: " +
                timelinePropertiesFile.canonicalPath

            return }

        this.timelineProperties = new TimelineProperties(timelinePropertiesFile)
        this.timeline = timelineProperties.timeline

        showTimeline(sin, out, err) }

    public void showTimeline(final def sin, def out, def err) {

        //out.println ""
        def currentMarker = timeline.getLastMarker(new Date())
        def reader = new NonBlockingInputStreamReader(sin)
        Thread readerThread = new Thread(reader)
        readerThread.start()

        boolean running = true

        def blockingReadLine = {
            String line = null;
            while (line == null && readerThread.isAlive()) {
                line = reader.readLine()

                Thread.sleep(200) }

            return line }

        def readNotes = {
            out.println(ansi().fg(YELLOW).
                a("Notes (end with EOF or a blank line):").reset())

            String notes = ""
            String line = null
            line = blockingReadLine()

            while(line != "" && line != "EOF" && line != null) {
                notes += line + EOL
                line = blockingReadLine() }

            return notes }

        def printPrompt = {
            out.print(formatMarker(currentMarker) + EOL +
                ansi().fg(YELLOW).a("> ").reset())
            out.flush() }

        String line = null

        printPrompt()
        while (running && readerThread.isAlive()) {

           // Handle user input
            line = reader.readLine()
            if (line != null) {
                out.flush();
                switch (line) {
                    case ~/quit|exit|\u0004/:
                        running = false;
                        break

                    case ~/n|new/:

                        // Read mark
                        out.println(ansi().fg(YELLOW).a("New timestamp:").reset())
                        String mark = blockingReadLine()

                        // Read notes
                        String notes = readNotes();

                        // Create marker
                        currentMarker = new TimelineMarker(new Date(), mark, notes)
                        timeline.addMarker(currentMarker)
                        if (timelineProperties.persistOnUpdate)
                            timelineProperties.save()
                        break

                    case ~/h|help/:
                        out.println(ansi().fg(RED).
                            a("Not yet implemented.").reset());
                        break

                    case ~/l|list|history/:
                        out.println(ansi().fg(RED).
                            a("Not yet implemented.").reset());
                        break

                    case ~/s|save/:
                        timelineProperties.save()
                        break

                    case ~/reload/:
                        timeline = timelineProperties.reloadTimeline()
                        currentMarker = timeline.getLastMarker(new Date())
                        break

                    default:
                        String notes = readNotes()
                        currentMarker = new TimelineMarker(new Date(), line, notes)
                        timeline.addMarker(currentMarker)
                        if (timelineProperties.persistOnUpdate)
                            timelineProperties.save()
                        break
                }
                printPrompt()
            } else {
                out.print(ansi().saveCursorPosition().cursorUpLine().eraseLine().toString() +
                    formatMarker(currentMarker) +
                    ansi().cursorDown(1).restorCursorPosition().toString())
                out.flush();

                Thread.sleep(200)
            }
        }

        if (readerThread.isAlive()) {
            readerThread.interrupt();
            readerThread.join(500);
            if (readerThread.isAlive()) readerThread.stop(); }

        out.println ""
    }

    protected static String formatMarker(TimelineMarker tm) {
        return ansi().fgBright(CYAN).
            a(Timeline.shortFormat.format(tm.timestamp) + " ").fg(GREEN).
            a("(" + getDuration(tm) + ") ").fg(WHITE).a(tm.mark).toString() }

    protected static String getDuration(TimelineMarker tm) {
        Date currentTime = new Date()
        long seconds = currentTime.time - tm.timestamp.time
        seconds /= 1000
        long minutes = seconds / 60
        seconds = seconds % 60
        long hours = minutes / 60
        minutes %= 60
        long days = hours / 24
        hours %= 24

        StringBuilder sb = new StringBuilder()
        if (days > 0) sb.append(days + "day ")
        if (hours > 0) sb.append(hours + "hr ")
        if (minutes > 0) sb.append(minutes + "min ")
        sb.append(seconds + "sec")

        return sb.toString() }

}
